//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 08/09/02; 15:10:44

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "graphic.h"
#include "menulib.h"
#include "mineswp.h"

//-----------------------------------------------------------------------------

extern TMenu * Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//-----------------------------------------------------------------------------

TGame * Game;

char * yesno_str[] = {"No","Yes"};

char * Plane0;
char * Plane1;

//-----------------------------------------------------------------------------

void InputStr (char *buffer, short maxlen)
{
	SCR_STATE ss;
	short key, i = 0;
	buffer[0] = 0;
	SaveScrState (&ss);
	do
	{
		SetPlane(0);
		MoveTo (ss.CurX, ss.CurY);
		printf ("%s_  ", buffer);
		SetPlane(1);
		MoveTo (ss.CurX, ss.CurY);
		printf ("%s_  ", buffer);
		// Note that two spaces are required if F_4x6 font is used
		key = ngetchx ();
		if (key >= ' ' && key <= '~' && i < maxlen) buffer[i++] = key;
		if (key == KEY_BACKSPACE && i) i--;
		buffer[i] = 0;
	} while (key != KEY_ENTER);
}
//--------------------------------------------------------------------------


void NewGame(void)
{
	int a;
	int x,y;
	
	// Creation
	
	
	for (x=0;x<Game->width;x++)
	for (y=0;y<Game->height;y++)
	{
		Game->Tiles[x][y].MinesAroundCount = 0;
		Game->Tiles[x][y].state = TS_Hidden;
	}
	
	
	for (a=0;a<Game->mines;a++)
	{
		
		do
		{
			x = random(Game->width);
			y = random(Game->height);
		}
		while (Game->Tiles[x][y].MinesAroundCount==MINE_NUM);
		
		AddMine(x,y);
	}
	
	
	Game->pos_x = Game->width/2;
	Game->pos_y = Game->height/2;
	
	Game->game_state = TG_Playing;
	
	Game->mines_counter = Game->mines;
	Game->viewed_counter = (Game->width*Game->height);
	
	
	
}
//-----------------------------------------------------------------------------

void RefreshTile(int x, int y)
{
	int xx,yy;
	TTile*TilePtr=&Game->Tiles[x][y];
	
	xx = (x<<3);
	yy = (y<<3);
	
	unsigned char *src0;
	unsigned char *src1;
	
	if (TilePtr->state==TS_Viewed)
	{
		src0=Numbers_C0+(TilePtr->MinesAroundCount<<3);
		src1=Numbers_C1+(TilePtr->MinesAroundCount<<3);
	}
	else
	{
		src0=Tiles_C0+(TilePtr->state<<3);
		src1=Tiles_C1+(TilePtr->state<<3);
	}
	
	GraySprite8_OR(xx,yy,8,src0,src1,Plane0,Plane1);
	
}
//-----------------------------------------------------------------------------

void RefreshScreen(void)
{
	int x,y;
	char buffer[50];
	
	
	ClearGrayScreen2B(Plane0,Plane1);
	
	// Cases
	
	for (x=0;x<Game->width;x++)
	for (y=0;y<Game->height;y++)
	RefreshTile(x,y);
	
	// Smiley
	
	GraySprite32_OR(STATS_LEFT,0,SMILEY_HEIGHT,
	Smileys_C0+SMILEY_HEIGHT*Game->game_state,
	Smileys_C1+SMILEY_HEIGHT*Game->game_state,
	Plane0,Plane1);
	
	// Stats
	
	sprintf(buffer,"%03d",Game->mines_counter);
	DrawGrayStr2B(STATS_LEFT,40,buffer,A_REPLACE,Plane0,Plane1);
	
	FastCopyScreen(Plane0,GetPlane(0));
	FastCopyScreen(Plane1,GetPlane(1));
	
}
//-----------------------------------------------------------------------------

void AddMine(int x, int y)
{
	int a,b;
	TTile*TilePtr=&Game->Tiles[x][y];
	
	TilePtr->MinesAroundCount = MINE_NUM;
	
	for (a=-1;a<=1;a++)
	for (b=-1;b<=1;b++)
	{
		TTile*TilePtr2=&Game->Tiles[x+a][y+b];
		
		if (IsPointIntoMap(x+a,y+b))
		if (!(a==0 && b==0) && TilePtr2->MinesAroundCount!=MINE_NUM )
		{
			TilePtr2->MinesAroundCount++;
		}
	}
	
}
//-----------------------------------------------------------------------------

void RemoveMine(int x, int y)
{
	int a,b;
	TTile*TilePtr=&Game->Tiles[x][y];
	
	TilePtr->MinesAroundCount = 0;
	
	for (a=-1;a<=1;a++)
	for (b=-1;b<=1;b++)
	{
		TTile*TilePtr2=&Game->Tiles[x+a][y+b];
		
		if (IsPointIntoMap(x+a,y+b))
		if (!(a==0 && b==0) && TilePtr2->MinesAroundCount!=MINE_NUM )
		{
			TilePtr2->MinesAroundCount--;
		}
	}
	
	for (a=-1;a<=1;a++)
	for (b=-1;b<=1;b++)
	{
		TTile*TilePtr2=&Game->Tiles[x+a][y+b];
		
		if (IsPointIntoMap(x+a,y+b))
		if (!(a==0 && b==0) && TilePtr2->MinesAroundCount==MINE_NUM )
		{
			TilePtr->MinesAroundCount++;
		}
	}
	
	
}
//-----------------------------------------------------------------------------


void LeftClick(int x, int y)
{
	int xx,yy;
	TTile*TilePtr=&Game->Tiles[x][y];
	
	if (TilePtr->state==TS_Hidden || TilePtr->state==TS_Wonder)
	{
		
		// First Time : Cannot lose
		
		if (Game->viewed_counter==Game->width*Game->height)
		{
			
			if (TilePtr->MinesAroundCount==MINE_NUM)
			{
				
				for (;;)
				{
					xx = random(Game->width);
					yy = random(Game->height);
					
					if (Game->Tiles[xx][yy].MinesAroundCount!=MINE_NUM)
					break;
				}
				
				RemoveMine(x,y);
				AddMine(xx,yy);
			}
		}
		
		if (TilePtr->MinesAroundCount==MINE_NUM)
		{
			
			// ***** LOSE *****
			
			for (xx=0;xx<Game->width;xx++)
			for (yy=0;yy<Game->height;yy++)
			{
				TTile*TilePtr2=&Game->Tiles[xx][yy];
				
				if (TilePtr2->state==TS_Mine && TilePtr2->MinesAroundCount!=MINE_NUM)
				TilePtr2->state = TS_WrongMine;
				else if (TilePtr2->state!=TS_Mine && TilePtr2->MinesAroundCount==MINE_NUM)
				TilePtr2->state = TS_MineShowed;
			}
			
			TilePtr->state = TS_MineShowedStart;
			
			Game->game_state = TG_Lose;
			
			
		}
		else
		ZeroPointExecute(x,y);
		
		// ***** WIN *****
		
		if (Game->viewed_counter==Game->mines)
		{
			
			Game->mines_counter = 0;
			
			for (xx=0;xx<Game->width;xx++)
			for (yy=0;yy<Game->height;yy++)
			{
				TTile*TilePtr2=&Game->Tiles[xx][yy];
				
				if (TilePtr2->state==TS_Hidden || TilePtr2->state==TS_Wonder)
				TilePtr2->state = TS_Mine;
			}
			
			Game->game_state = TG_Win;
			
		}
		
		
	}
	
	
}
//-----------------------------------------------------------------------------

void RightClick(int x, int y)
{
	
	TTile*TilePtr=&Game->Tiles[x][y];
	
	if (TilePtr->state!=TS_Viewed)
	{
		if (TilePtr->state==TS_Mine)
		Game->mines_counter++;
		
		if (TilePtr->state<2)
		TilePtr->state++;
		else
		TilePtr->state = 0;
		
		
		if (!Game->use_marks && TilePtr->state==TS_Wonder)
		TilePtr->state = 0;
		
		if (TilePtr->state==TS_Mine)
		Game->mines_counter--;
		
	}
	
	
}
//-----------------------------------------------------------------------------

void RLClick(int x, int y)
{
	int a,b,c;
	TTile*TilePtr=&Game->Tiles[x][y];
	
	
	if (TilePtr->state==TS_Viewed)
	{
		
		c = 0;
		
		for (a=-1;a<=1;a++)
		for (b=-1;b<=1;b++)
		{
			TTile*TilePtr2=&Game->Tiles[x+a][y+b];
			
			if (IsPointIntoMap(x+a,y+b))
			if (!(a==0 && b==0) && TilePtr2->state==TS_Mine )
			{
				c++;
			}
		}
		
		if (TilePtr->MinesAroundCount==c)
		{
			for (a=-1;a<=1;a++)
			for (b=-1;b<=1;b++)
			{
				TTile*TilePtr2=&Game->Tiles[x+a][y+b];
				
				if (IsPointIntoMap(x+a,y+b))
				if (!(a==0 && b==0) && TilePtr2->state==TS_Hidden )
				{
					LeftClick(x+a,y+b);
				}
			}
			
		}
		
	}
	
}
//-----------------------------------------------------------------------------


void CorrectOptions(void)
{
	
	if (Game->width>WIDTH_MAX)
	Game->width = 8;
	if (Game->width<8)
	Game->width = WIDTH_MAX;
	
	if (Game->height>HEIGHT_MAX)
	Game->height = 8;
	if (Game->height<8)
	Game->height = HEIGHT_MAX;
	
	int mines_max=(Game->width-1)*(Game->height-1);
	
	if (Game->mines<10)
	Game->mines = mines_max;
	if (Game->mines>mines_max)
	Game->mines = 10;
	
}
//-----------------------------------------------------------------------------

int IsPointIntoMap(int x, int y)
{
	return (x>=0 && x<Game->width && y>=0 && y<Game->height);
}
//-----------------------------------------------------------------------------


void ZeroPointAround(int x, int y)
{
	int a,b;
	TTile*TilePtr=&Game->Tiles[x][y];
	
	
	
	TilePtr->state = TS_Viewed;
	Game->viewed_counter--;
	
	//RefreshScreen();
	//ngetchx();
	
	if (TilePtr->MinesAroundCount==0)
	{
		
		for (a=-1;a<=1;a++)
		for (b=-1;b<=1;b++)
		{
			TTile*TilePtr2=&Game->Tiles[x+a][y+b];
			
			if (IsPointIntoMap(x+a,y+b))
			if (!(a==0 && b==0)
			&& TilePtr2->MinesAroundCount!=MINE_NUM 
			&& (TilePtr2->state==TS_Hidden || TilePtr2->state==TS_Wonder))
			ZeroPointAdd(x+a,y+b);
			
			
		}
		
	}
	
}
//-----------------------------------------------------------------------------

void ZeroPointAdd(int x, int y)
{
	
	int a;
	
	if (Game->ZeroPointsCount)
	for (a=0;a<Game->ZeroPointsCount;a++)
	if (x==Game->ZeroPoints[a].x && y==Game->ZeroPoints[a].y)
	return;
	
	Game->ZeroPoints[Game->ZeroPointsCount].x=x;
	Game->ZeroPoints[Game->ZeroPointsCount].y=y;
	Game->ZeroPointsCount++;
	
}
//-----------------------------------------------------------------------------

void ZeroPointExecute(int x, int y)
{
	
	Game->ZeroPointsCount=0;
	ZeroPointAround(x,y);
	
	while (Game->ZeroPointsCount)
	{
		ZeroPointAround(Game->ZeroPoints[0].x,Game->ZeroPoints[0].y);
		Game->ZeroPoints[0]=Game->ZeroPoints[Game->ZeroPointsCount-1];
		Game->ZeroPointsCount--;
	}
	
}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{
	short key;
	char buffer[50];
	int xx,yy;
	
	if (!(Plane0=malloc(2*LCD_SIZE)))
	goto end_prog;
	Plane1 = Plane0+LCD_SIZE;
	
	TMenu MenuA;
	Menu = &MenuA;
	
	TGame GameA;
	Game=&GameA;
	
	randomize();
	
	// OPTIONS
	Game->width=8;
	Game->height=8;
	Game->mines=10;
	Game->use_marks = 1;
	
	CorrectOptions();
	
	
	for (;;)
	{
		
		MenuNew2();
		MenuAddTitle("Minesweeper");
		MenuAddItem("Play");
		MenuAddItem("Options");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		MenuDo(ME_MAIN);
		
		if (!Menu->ok) break;
		
		if (Menu->Index==1)
		{
			
			GrayOn();
			
			NewGame();
			RefreshScreen();
			
			for (;;)
			{
				
				xx = Game->pos_x*8;
				yy = Game->pos_y*8;
				
				GraySprite8_XOR(xx,yy,8,Cursor_C0,Cursor_C0,GetPlane(0),GetPlane(1));
				key = ngetchx();
				GraySprite8_XOR(xx,yy,8,Cursor_C0,Cursor_C0,GetPlane(0),GetPlane(1));
				
				
				
				if (key==KEY_LEFT)
				{
					Game->pos_x--;
					if (Game->pos_x<0)
					Game->pos_x = (Game->width-1);
				}
				
				else if (key==KEY_RIGHT)
				{
					Game->pos_x++;
					if (Game->pos_x>=Game->width)
					Game->pos_x = 0;
				}
				
				else if (key==KEY_UP)
				{
					Game->pos_y--;
					if (Game->pos_y<0)
					Game->pos_y = (Game->height-1);
				}
				
				else if (key==KEY_DOWN)
				{
					Game->pos_y++;
					if (Game->pos_y>=Game->height)
					Game->pos_y = 0;
				}
				
				else if (key==KEY_F1 && Game->game_state==TG_Playing)
				{
					LeftClick(Game->pos_x,Game->pos_y);
					RefreshScreen();
				}
				
				else if (key==KEY_F2 && Game->game_state==TG_Playing)
				{
					RightClick(Game->pos_x,Game->pos_y);
					RefreshScreen();
				}
				
				else if (key==KEY_F3 && Game->game_state==TG_Playing)
				{
					RLClick(Game->pos_x,Game->pos_y);
					RefreshScreen();
				}
				
				else if (key==KEY_F4)
				{
					NewGame();
					RefreshScreen();
				}
				
				else if (key==KEY_APPS)
				{
					off();
				}
				
				else if (key==KEY_ESC)
				{
					
					GrayOff();
					clrscr();
					printf("[ESC]   : Continue \n");
					printf("[ENTER] : Exit \n");
					
					while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
					
					GrayOn();
					
					if (key==KEY_ENTER)
					break;
					
					RefreshScreen();
					
				}
				
			}
			
			GrayOff();
			
			
		}
		
		else if (Menu->Index==2)
		{
			
			for (;;)
			{
				
				CorrectOptions();
				
				MenuNew2();
				MenuAddTitle("Options");
				sprintf(buffer,"Width : %d",Game->width);
				MenuAddItem(buffer);
				sprintf(buffer,"Height : %d",Game->height);
				MenuAddItem(buffer);
				sprintf(buffer,"Mines : %d",Game->mines);
				MenuAddItem(buffer);
				sprintf(buffer,"Marks : %s",yesno_str[Game->use_marks]);
				MenuAddItem(buffer);
				MenuAddItem("Enter Mines Number");
				
				MenuDo(ME_OPTIONS);
				
				if (!Menu->ok) break;
				
				if (Menu->Index==1)
				Game->width++;
				else if (Menu->Index==2)
				Game->height++;
				else if (Menu->Index==3)
				Game->mines++;
				else if (Menu->Index==4)
				Game->use_marks^=1;
				else if (Menu->Index==5)
				{
					clrscr();
					printf("Mines Number (10-%d): \n",(Game->width-1)*(Game->height-1));
					InputStr(buffer,3);
					Game->mines=atoi(buffer);
				}
				
			}
			
			
		}
		
		else if (Menu->Index==3)
		{
			clrscr();
			FontSetSys(0);
			printf("About \n");
			printf("MineSweeper 1.2 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
		}
		
		else if (Menu->Index==4)
		{
			break;
		}
		
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
}
